/* treewm - an X11 window manager.
 * Copyright (c) 2000 Thomas Jger <thehunter2000@web.de>
 * This code is released under the terms of the GNU GPL. See
 * the included file LICENSE for details.
 */

#include "clientinfo.h"
#include <string.h>

ClientInfo::ClientInfo(Section *section) {
  s = section;
}

void ClientInfo::Init() {
  char *c;
  int v;
  flags = 0;
  c = s ? s->GetEntry("Name") : 0;
  if (c) {
    name = new char[strlen(c)+1];
    strcpy(name,c);
  } else name = 0;
  if (s && s->GetIntEntry("X",v)) {
    flags |= CI_X;
    x = v;
  }
  if (s && s->GetIntEntry("Y",v)) {
    flags |= CI_Y;
    y = v;
  }
  if (s && s->GetIntEntry("Width",v)) {
    flags |= CI_WIDTH;
    width = v;
  }
  if (s && s->GetIntEntry("Height",v)) {
    flags |= CI_HEIGHT;
    height = v;
  }
  if (s && s->GetIntEntry("XIcon",v)) {
    flags |= CI_ICONX;
    IconX = v;
  }
  if (s && s->GetIntEntry("XIcon",v)) {
    flags |= CI_ICONY;
    IconY = v;
  }
  if (s && s->GetIntEntry("VirtualX",v) && v>=1) {
    flags |= CI_VIRTUALX;
    VirtualX = v;
  }
  if (s && s->GetIntEntry("VirtualY",v) && v>=1) {
    flags |= CI_VIRTUALY;
    VirtualY = v;
  }
  if (s && s->GetIntEntry("AutoClose",v))
    if (v) {
      flags |= CI_AUTOCLOSE;
    }
  if (s && s->GetIntEntry("Maximized",v))
    if (v) {
      flags |= CI_MAXIMIZED;
      if (v > 1)
        flags |= CI_MMAXIMIZED;
    }
  if (s && s->GetIntEntry("Sticky",v) && v)
    flags |= CI_STICKY;
  if (s && s->GetIntEntry("AutoSroll",v) && v)
    flags |= CI_AUTOSCROLL;
  if (!s || !(s->GetIntEntry("HasTitle",v)) || v)
    flags |= CI_HASTITLE;
  if (!s || !(s->GetIntEntry("HasTaskBarEntry",v)) || v)
    flags |= CI_HASTBENTRY;
  c = s ? s->GetEntry("IconPath") : 0;
  if (c) {
    iconpm = rman->GetPixmap(c);
  } else iconpm = 0;

  c = s ? s->GetEntry("BGPixmap") : 0;
  if (c) {
    bgpm = rman->GetPixmap(c);
  } else bgpm = 0;

  c = s ? s->GetEntry("Sceme") : 0;
  if (c) {
    lower(c);
    Sc = (Sceme *)rman->GetInfo(SE_SCEME,c);
  } else
    Sc = 0;

  if (s && s->GetIntEntry("Target",v)) {
    target = v;
  } else target = -1;

  stacking = 1;
  if (s && s->GetIntEntry("Stacking",v)) {
    if (v >= 2) {
      stacking = 2;
    } else {
      if (v <= 0) {
        stacking = 0;
      }
    }
  }
  if (s && s->GetIntEntry("Tile",v) && v)
    flags |= CI_TILE;

  if (s && s->GetIntEntry("NoBackGround",v) && v)
    flags |= CI_NOBACKGROUND;


  actions = 0;
  ActionList *last=0;
  Action *a;
  for (Entry *e=s->fe;e;e=e->next)
    if (!strncasecmp("Action",e->tag,6)) {
      lower(e->value);
      a = (Action *)rman->GetInfo(SE_ACTION,e->value);
      // Add action
      if (a) {
        if (last) {
          last->next = new ActionList;;
          last = last->next;
        } else {
          actions = new ActionList;
          last = actions;
        }
        last->a = a;
        last->next = 0;
      }
    }

}

ClientInfo::~ClientInfo() {
  ActionList *as;
  for (;actions;actions=as) {
    as = actions->next;
    delete actions;
  }
  delete [] name;
}
